use crate::client::ONLINE_USERS;
use crate::events::{notify_all_clients_to_update_online_users, Event, CLIENT_SENDERS};
use crate::handles::handshake::HOST_ENABLED_ID_SET;
use crate::handles::redis::update_client_redis_data;
use crate::typing::message_typed::ClientMessageData;
use crate::utils::json_utils::get_current_timestamp;
use dashmap::DashMap;
use lazy_static::lazy_static;
use std::time::Duration;
use tokio::sync::mpsc;
use tokio::sync::mpsc::UnboundedSender;
use tokio::time::sleep;

lazy_static! {
    pub static ref refuse_procedure_map: DashMap<String, UnboundedSender<()>> = DashMap::new();
    pub static ref channel_hangup_procedure_map: DashMap<String, UnboundedSender<()>> =
        DashMap::new();
}

async fn send_inside_message(
    target_sender: &(String, u128),
    event_sender: &UnboundedSender<Event>,
    json_message: String,
    from_id: &String,
) {
    println!("发送给用户id {} 的消息 {}", from_id, json_message);
    if let Err(e) = event_sender.send(Event::SendClientMessage(
        target_sender.clone(),
        json_message,
        false,
    )) {
        println!("发送给用户id {} 独立消息失败：{:?}", from_id, e);
    } else {
        println!("发送给用户id {} 独立消息成功", from_id);
    }
}

pub async fn handle_agora_call(
    client_message_data: &ClientMessageData,
    from_id: &String,
    event_sender: &UnboundedSender<Event>,
    connection_time: &u128,
) {
    let target_sender_which = (from_id.to_string(), connection_time.clone());
    match client_message_data.msg_type.as_str() {
        "CancelCall" => {
            let calling_to_id = &client_message_data.to_id;
            println!(
                "收到客户端取消呼叫 取消呼叫组: {} 呼叫方id {}",
                calling_to_id,
                from_id.to_string()
            );
            let hangup_message_json = serde_json::json!({
                "msgType": "CmdCancelCall",
                "fromID": "0",
                "fromName": "Server",
                "toID": from_id,
                "msgData": {
                    "channelId": "",
                    "rtcToken": ""
                }
            })
                .to_string();
            if let Err(e) = event_sender.send(Event::SendClientMessage(
                target_sender_which.clone(),
                hangup_message_json,
                false,
            )) {
                println!("发送给用户id {} 独立消息失败：{:?}", from_id, e);
            }
            let user_status = ONLINE_USERS
                .get(from_id)
                .map(|v| v.split(',').map(|s| s.to_string()).collect::<Vec<_>>());
            if let Some(mut the_current_user_data) = user_status {
                if the_current_user_data[1].is_empty() {
                    the_current_user_data[1] = "".to_string();
                    the_current_user_data[0] = "idle".to_string();
                    the_current_user_data[6] = "0".to_string();
                    let data_joined = the_current_user_data.join(",");
                    ONLINE_USERS.insert(from_id.to_string(), data_joined.clone());
                    let from_id_clone = from_id.clone();
                    tokio::spawn(async move {
                        if let Err(e) = update_client_redis_data(&from_id_clone, data_joined).await {
                            println!("更新redis数据失败：{:?}", e);
                        }
                    });
                    tokio::spawn(async move {
                        notify_all_clients_to_update_online_users().await;
                    });
                    return;
                }
                let all_to_hangup_id_vec: Vec<String> =
                    calling_to_id.split(',').map(|s| s.to_string()).collect();
                for to_hangup_id in all_to_hangup_id_vec {
                    if let Some(ref_procedure_sender) = refuse_procedure_map.get(&to_hangup_id) {
                        if let Err(e) = ref_procedure_sender.send(()) {
                            println!("取消拒绝接听的线程队列失败：{:?}", e);
                        } else {
                            println!("取消拒绝接听的线程队列成功");
                        }
                    }
                    let the_other_caller_data = ONLINE_USERS
                        .get(&to_hangup_id)
                        .map(|v| v.split(',').map(|s| s.to_string()).collect::<Vec<_>>());
                    if let Some(mut the_other_caller_data) = the_other_caller_data {
                        if the_other_caller_data[1] == the_current_user_data[1] {
                            let hangup_message_json = serde_json::json!({
                                "msgType": "CmdHangup",
                                "fromID": "0",
                                "fromName": "Server",
                                "toID": to_hangup_id,
                                "msgData": {
                                    "channelId": "",
                                    "rtcToken": ""
                                }
                            })
                                .to_string();
                            let target_sender = CLIENT_SENDERS
                                .iter()
                                .find(|entry| entry.key().0 == *calling_to_id)
                                .map(|entry| entry.key().clone());
                            if let Some(target_sender) = target_sender {
                                if let Err(e) = event_sender.send(Event::SendClientMessage(
                                    target_sender,
                                    hangup_message_json,
                                    false,
                                )) {
                                    println!("发送给用户id {} 独立消息失败：{:?}", to_hangup_id, e);
                                } else {
                                    the_other_caller_data[0] = "idle".to_string();
                                    the_other_caller_data[1] = "".to_string();
                                    the_other_caller_data[6] = "0".to_string();
                                    let all_data_joined = the_other_caller_data.join(",");
                                    println!("step - 0");
                                    ONLINE_USERS.insert(to_hangup_id.clone(), all_data_joined.clone());
                                    println!("step - 1");
                                    tokio::spawn(async move {
                                        if let Err(e) =
                                            update_client_redis_data(&to_hangup_id, all_data_joined)
                                                .await
                                        {
                                            println!("更新redis数据失败：{:?}", e);
                                        }
                                    });
                                }
                            } else {
                                println!("未找到对应的sender");
                            }
                        }
                    }
                }
                println!("step - 1.5");
                the_current_user_data[1] = "".to_string();
                the_current_user_data[0] = "idle".to_string();
                the_current_user_data[6] = "0".to_string();
                let the_current_data_joined = the_current_user_data.join(",");
                println!("step - 1.7");
                ONLINE_USERS.insert(from_id.to_string(), the_current_data_joined.clone());
                let from_id_clone = from_id.clone();
                println!("step - 2.1");
                tokio::spawn(async move {
                    if let Err(e) =
                        update_client_redis_data(&from_id_clone, the_current_data_joined).await
                    {
                        println!("更新redis数据失败：{:?}", e);
                    }
                });
                tokio::spawn(async move {
                    println!("更新在线用户列表 Cancel-Call");
                    notify_all_clients_to_update_online_users().await;
                });
                println!("step - 2.2");
            }
        }
        "Call" => {
            let calling_to_id = &client_message_data.to_id;
            println!(
                "收到客户端呼叫 呼叫方id {} 呼叫组id {}",
                from_id, calling_to_id
            );
            if calling_to_id.is_empty() {
                let error_json = format!(
                    r#"{{"msgType": "Error", "fromID": "0", "fromName": "Server", "msgData": "请指定呼叫对象", "toID": "{}"}}"#,
                    from_id
                );
                send_inside_message(&target_sender_which, event_sender, error_json, from_id).await;
                return;
            }
            if calling_to_id.eq(from_id) {
                let error_self_json = format!(
                    r#"{{"msgType": "Error", "fromID": "0", "fromName": "Server", "msgData": "不能给自己打电话", "toID": "{}"}}"#,
                    from_id
                );
                send_inside_message(&target_sender_which, event_sender, error_self_json, from_id)
                    .await;
                return;
            }
            let user_status = ONLINE_USERS
                .get(from_id)
                .map(|v| v.split(',').map(|s| s.to_string()).collect::<Vec<_>>());
            if let Some(mut the_user_data) = user_status {
                let channel_id_now = if the_user_data[1].is_empty() {
                    format!("{}-{}", from_id, get_current_timestamp())
                } else {
                    the_user_data[1].to_string()
                };
                let calling_to_id_vec: Vec<&str> = calling_to_id.split(',').collect();
                let mut refuse_thread_users: Vec<String> = vec![];
                println!(
                    "当前用户channelId {} 呼叫方id集合是 {:?}",
                    channel_id_now, calling_to_id_vec
                );
                for calling_to_id in calling_to_id_vec {
                    if !ONLINE_USERS.contains_key(calling_to_id) {
                        let json_string = format!(
                            r#"{{"msgType":"Error","fromID":"0","fromName":"Server","msgData":"对方不在线","toID":"{}"}}"#,
                            client_message_data.to_id
                        );
                        send_inside_message(&target_sender_which, event_sender, json_string, from_id)
                            .await;
                        continue;
                    }
                    let user_status = ONLINE_USERS
                        .get(calling_to_id)
                        .map(|v| v.split(',').map(|s| s.to_string()).collect::<Vec<_>>());
                    if let Some(mut the_another_user_data) = user_status {
                        if the_another_user_data[0] != "idle" {
                            let json_string = format!(
                                r#"{{"msgType":"Error","fromID":"0","fromName":"Server","msgData":"对方正在呼叫中","toID":"{}"}}"#,
                                from_id
                            );
                            send_inside_message(&target_sender_which, event_sender, json_string, from_id)
                                .await;
                            continue;
                        }
                        let call_message_json = serde_json::json!({
                            "msgType": "Call",
                            "fromID": from_id,
                            "fromName": client_message_data.from_name,
                            "toID": calling_to_id,
                            "msgData": {
                                "channelId": channel_id_now,
                                "rtcToken": ""
                            }
                        })
                            .to_string();
                        let target_sender = CLIENT_SENDERS
                            .iter()
                            .find(|entry| entry.key().0 == *calling_to_id)
                            .map(|entry| entry.key().clone());
                        if let Some(sender) = target_sender {
                            if let Err(e) = event_sender.send(Event::SendClientMessage(
                                sender.clone(),
                                call_message_json.clone(),
                                false,
                            )) {
                                println!("发送给用户id {:?} 独立消息失败：{:?}", sender, e);
                                continue;
                            } else {
                                println!(
                                    "发送给用户id {} 独立消息成功 消息内容 {}",
                                    calling_to_id, call_message_json
                                );
                            }
                        } else {
                            println!("找不到对应的sender，无法发送客户端消息！");
                            continue;
                        }
                        println!("发送给用户id {} 独立消息成功", calling_to_id);
                        the_another_user_data[0] = "callin".to_string();
                        the_another_user_data[1] = channel_id_now.clone();
                        the_another_user_data[6] = "0".to_string();
                        println!("step - 1");
                        let the_another_user_data_joined = the_another_user_data.join(",");
                        ONLINE_USERS.insert(calling_to_id.to_string(), the_another_user_data_joined.clone());
                        println!("step - 2");
                        let calling_id_c = calling_to_id.to_string();
                        tokio::spawn(async move {
                            if let Err(e) = update_client_redis_data(
                                &calling_id_c,
                                the_another_user_data_joined,
                            )
                                .await
                            {
                                println!("更新redis数据失败：{:?} 用户id {}", e, calling_id_c);
                            } else {
                                println!("更新redis数据成功");
                            }
                        });
                        println!("step - 3");
                        refuse_thread_users.push(calling_to_id.to_string());
                    }
                }
                println!(
                    "是否通知其他用户成功？ {} 然后发送CmdCall？",
                    refuse_thread_users.len()
                );
                if refuse_thread_users.len() > 0 {
                    println!("发送给用户id {} 需要发送CmdCall", from_id);
                    if the_user_data[0] == "idle" || the_user_data[0] == "calling" {
                        let cmd_call_message_json = serde_json::json!({
                            "msgType": "CmdCall",
                            "fromID": calling_to_id,
                            "fromName": client_message_data.from_name,
                            "toID": from_id,
                            "msgData": {
                                "channelId": channel_id_now,
                                "rtcToken": ""
                            }
                        })
                            .to_string();
                        the_user_data[0] = if the_user_data[0] == "calling" {
                            "calling".to_string()
                        } else {
                            "callout".to_string()
                        };
                        the_user_data[1] = channel_id_now.clone();
                        the_user_data[6] = "0".to_string();
                        let the_user_data_joined = the_user_data.join(",");
                        println!("step - 4");
                        ONLINE_USERS.insert(from_id.to_string(), the_user_data_joined.clone());
                        let from_id_c = from_id.clone();
                        println!("step - 5");
                        tokio::spawn(async move {
                            if let Err(e) =
                                update_client_redis_data(&from_id_c, the_user_data_joined).await
                            {
                                println!("更新redis数据失败：{:?} 用户id {}", e, from_id_c);
                            } else {
                                println!("更新redis数据成功");
                            }
                        });
                        println!("step - 6");
                        send_inside_message(
                            &target_sender_which,
                            event_sender,
                            cmd_call_message_json,
                            from_id,
                        )
                            .await;
                        tokio::spawn(async move {
                            notify_all_clients_to_update_online_users().await;
                        });
                        println!(
                            "step - 7 开启定时挂断线程refuse {}",
                            refuse_thread_users.len()
                        );
                        for user_id in refuse_thread_users {
                            let event_sender_clone = event_sender.clone();
                            let user_id_clone = user_id.clone();
                            let from_id_clone = from_id.clone();
                            let (cancel_tx, mut cancel_rx) = mpsc::unbounded_channel::<()>();
                            refuse_procedure_map.insert(user_id.clone(), cancel_tx);
                            println!("step -8 Refuse-Call auto");
                            tokio::spawn(async move {
                                tokio::select! {
                                    _ = sleep(Duration::from_secs(15)) => {
                                        println!("20秒内没有接听，准备挂断通话");
                                        let user_data = ONLINE_USERS.get(&user_id_clone).map(|v| v.split(',').map(|s| s.to_string()).collect::<Vec<_>>());
                                        if let Some(mut current_user_data_vec) = user_data {
                                            let current_channel_id = current_user_data_vec[1].clone();
                                            current_user_data_vec[0] = "idle".to_string();
                                            current_user_data_vec[1] = "".to_string();
                                            current_user_data_vec[6] = "0".to_string();
                                            println!("step R-H 1");
                                            ONLINE_USERS.insert(user_id_clone.clone(), current_user_data_vec.join(","));
                                            println!("step R-H 2");
                                            let user_id_clone_for = user_id_clone.clone();
                                            tokio::spawn(async move {
                                                if let Err(e) = update_client_redis_data(
                                                    &user_id_clone_for,
                                                    current_user_data_vec.join(","),
                                                )
                                                .await {
                                                    println!("更新redis数据失败：{:?} 用户id {}", e, user_id_clone_for);
                                                } else {
                                                    println!("通知挂断refuse线程，更新redis数据成功");
                                                }
                                            });
                                            println!("step R-H 3");
                                            let host_user_sender_which = CLIENT_SENDERS
                                                .iter()
                                                .find(|entry| entry.key().0 == from_id_clone)
                                                .map(|entry| entry.key().clone());
                                            println!("step R-H 4");
                                            if let Some(host_user_sender_which) = host_user_sender_which {
                                                send_inside_message(
                                                    &host_user_sender_which,
                                                    &event_sender_clone,
                                                    serde_json::json!({
                                                        "msgType": "CmdRefuse",
                                                        "fromID": user_id_clone,
                                                        "fromName": "Unknown",
                                                        "toID": host_user_sender_which.0,
                                                        "msgData": {
                                                            "channelId": "",
                                                            "rtcToken": ""
                                                        }
                                                    }).to_string(),
                                                    &host_user_sender_which.0,
                                                )
                                                .await;
                                            }
                                            println!("step R-H 5");
                                            let hangup_user_sender_which = CLIENT_SENDERS
                                                .iter()
                                                .find(|entry| entry.key().0 == user_id_clone)
                                                .map(|entry| entry.key().clone());
                                            println!("step R-H 6");
                                            if let Some(hangup_user_sender_which) = hangup_user_sender_which {
                                                send_inside_message(
                                                    &hangup_user_sender_which,
                                                    &event_sender_clone,
                                                    serde_json::json!({
                                                        "msgType": "CmdHangup",
                                                        "fromID": "0",
                                                        "fromName": "Server",
                                                        "toID": user_id_clone,
                                                        "msgData": {
                                                            "channelId": "",
                                                            "rtcToken": ""
                                                        }
                                                    }).to_string(),
                                                    &user_id_clone,
                                                )
                                                .await;
                                            }
                                            println!("step R-H 7");
                                            let left_users = ONLINE_USERS
                                                .iter()
                                                .filter(|entry| entry.value().split(',').collect::<Vec<_>>()[1] == current_channel_id)
                                                .map(|entry| (entry.key().clone(), entry.value().clone()))
                                                .collect::<Vec<_>>();
                                            println!("step R-H 8");
                                            if left_users.len() <= 1 {
                                                if let Some((user_id, user_data)) = left_users.get(0) {
                                                    let mut user_data_vec = user_data.split(',').map(|s| s.to_string()).collect::<Vec<_>>();
                                                    let sender_or_not = CLIENT_SENDERS
                                                        .iter()
                                                        .find(|entry| entry.key().0 == *user_id)
                                                        .map(|entry| entry.key().clone());
                                                    println!("step R-H 9");
                                                    if let Some(host_user_sender_which) = sender_or_not {
                                                        user_data_vec[0] = "idle".to_string();
                                                        user_data_vec[1] = "".to_string();
                                                        user_data_vec[6] = "0".to_string();
                                                        println!("step R-H 10");
                                                        ONLINE_USERS.insert(user_id.clone(), user_data_vec.join(","));
                                                        println!("step R-H 11");
                                                        let user_data_j = user_data_vec.join(",");
                                                        let uid = user_id.clone();
                                                        tokio::spawn(async move {
                                                            if let Err(e) = update_client_redis_data(&uid, user_data_j).await {
                                                                println!("只有一人在会议中，更新redis数据失败: {}", e);
                                                            } else {
                                                                println!("只有一人在会议中，更新redis数据成功");
                                                            }
                                                        });
                                                        println!("step R-H 12");
                                                        send_inside_message(
                                                            &host_user_sender_which,
                                                            &event_sender_clone,
                                                            serde_json::json!({
                                                                "msgType": "CmdHangup",
                                                                "fromID": "0",
                                                                "fromName": "Server",
                                                                "toID": host_user_sender_which.0,
                                                                "msgData": {}
                                                            }).to_string(),
                                                            user_id,
                                                        )
                                                        .await;
                                                    }
                                                }
                                            }
                                            println!("step R-H 13");
                                            tokio::spawn(async move {
                                                notify_all_clients_to_update_online_users().await;
                                            });
                                        }
                                    }
                                    _ = cancel_rx.recv() => {
                                        println!("用户已接听或者主动挂断或者通话被取消 {}", user_id_clone);
                                    }
                                }
                            });
                        }
                    }
                }
            }
        }
        "Refuse" => {
            println!("用户拒接电话 {}", from_id);
            if let Some(ref_procedure_sender) = refuse_procedure_map.get(from_id) {
                if let Err(e) = ref_procedure_sender.send(()) {
                    println!("取消拒绝接听的线程队列失败：{:?}", e);
                } else {
                    println!("取消拒绝接听的线程队列成功");
                }
            }
            let user_data = ONLINE_USERS
                .get(from_id)
                .map(|v| v.split(',').map(|s| s.to_string()).collect::<Vec<_>>());
            let mut current_channel_id = String::new();
            if let Some(mut current_user_data_vec) = user_data {
                let hangup_refuse_message = serde_json::json!({
                    "msgType": "CmdHangup",
                    "fromID": "0",
                    "fromName": "Server",
                    "toID": from_id,
                    "msgData": {
                        "channelId": "",
                        "rtcToken": ""
                    }
                })
                    .to_string();
                println!("step - Refuse - 0");
                current_channel_id = current_user_data_vec[1].clone();
                current_user_data_vec[0] = "idle".to_string();
                current_user_data_vec[1] = "".to_string();
                current_user_data_vec[6] = "0".to_string();
                println!("step - Refuse - 1");
                let current_user_data_joined = current_user_data_vec.join(",");
                println!("step - Refuse - 2");
                ONLINE_USERS.insert(from_id.to_string(), current_user_data_joined.clone());
                let from_id_clone = from_id.clone();
                println!("step - Refuse - 3");
                tokio::spawn(async move {
                    if let Err(e) =
                        update_client_redis_data(&from_id_clone, current_user_data_joined).await
                    {
                        println!("更新redis数据失败：{:?} 用户id {}", e, from_id_clone);
                    } else {
                        println!("更新redis数据成功");
                    }
                });
                println!("step - Refuse - 4");
                send_inside_message(
                    &target_sender_which,
                    event_sender,
                    hangup_refuse_message,
                    from_id,
                )
                    .await;
            }
            println!("step - Refuse - 5");
            let sender_found = CLIENT_SENDERS
                .iter()
                .find(|entry| entry.key().0 == client_message_data.to_id)
                .map(|entry| entry.key().clone());
            println!("step - Refuse - 6");
            if let Some(target_sender) = sender_found {
                let cmd_refuse_message = serde_json::json!({
                    "msgType": "CmdRefuse",
                    "fromID": from_id,
                    "fromName": "Server",
                    "toID": client_message_data.to_id,
                    "msgData": {
                        "channelId": "",
                        "rtcToken": ""
                    }
                })
                    .to_string();
                send_inside_message(
                    &target_sender,
                    event_sender,
                    cmd_refuse_message,
                    &client_message_data.to_id,
                )
                    .await;
            } else {
                println!("找不到toID对应的sender");
            }
            println!("step - Refuse - 7");
            let current_all_chatters = ONLINE_USERS
                .iter()
                .filter(|entry| entry.value().split(',').nth(1).unwrap_or("") == current_channel_id)
                .map(|entry| entry.key().clone())
                .collect::<Vec<_>>();
            if current_all_chatters.len() == 1 {
                let sender_found = CLIENT_SENDERS
                    .iter()
                    .find(|entry| entry.key().0 == current_all_chatters[0])
                    .map(|entry| entry.key().clone());
                if let Some(sender) = sender_found {
                    println!("step - Refuse - 7.75 - alone");
                    let cmd_cancel_call_message = serde_json::json!({
                        "msgType": "CmdCancelCall",
                        "fromID": "0",
                        "fromName": "Server",
                        "toID": current_all_chatters[0],
                        "msgData": {
                            "channelId": "",
                            "rtcToken": ""
                        }
                    })
                        .to_string();
                    send_inside_message(
                        &sender,
                        event_sender,
                        cmd_cancel_call_message,
                        &current_all_chatters[0],
                    )
                        .await;
                    println!("step - Refuse - 7.755 - alone");
                    let target_id = current_all_chatters[0].clone();
                    let target_ud = ONLINE_USERS
                        .get(&target_id)
                        .map(|v| v.split(',').map(|s| s.to_string()).collect::<Vec<_>>());
                    if let Some(mut target_ud_vec) = target_ud {
                        target_ud_vec[0] = "idle".to_string();
                        target_ud_vec[1] = "".to_string();
                        target_ud_vec[6] = "0".to_string();
                        let target_ud_joined = target_ud_vec.join(",");
                        println!("step - Refuse - 7.7599 - alone");
                        ONLINE_USERS.insert(target_id.clone(), target_ud_joined.clone());
                        println!("step - Refuse - 7.759999 - alone");
                        tokio::spawn(async move {
                            if let Err(e) = update_client_redis_data(&target_id, target_ud_joined).await {
                                println!("更新redis数据失败：{:?} 用户id {}", e, target_id);
                            } else {
                                println!("更新redis数据成功");
                            }
                        });
                    }
                }
            }
            println!("step - Refuse - 7.8");
            tokio::spawn(async move {
                notify_all_clients_to_update_online_users().await;
            });
            println!("step - Refuse - 8");
        }
        "EndMeeting" => {
            println!("step - EndMeeting - 1 收到挂断会议通知！");
            let user_found = ONLINE_USERS
                .get(from_id)
                .map(|v| v.split(',').map(|s| s.to_string()).collect::<Vec<_>>());
            if let Some(current_user_data_vec) = user_found {
                println!("step - EndMeeting - 1.1 判断是否为主持人");
                if current_user_data_vec[6] != "1" {
                    println!("step - EndMeeting - 1.2 不是主持人");
                    let error_message = serde_json::json!({
                        "msgType": "Error",
                        "fromID": "0",
                        "fromName": "Server",
                        "toID": from_id,
                        "msgData": "只有主持人可以结束会议！"
                    })
                        .to_string();
                    send_inside_message(&target_sender_which, event_sender, error_message, from_id)
                        .await;
                    println!("step - EndMeeting - 1.3 不是主持人消息发送完毕");
                } else {
                    println!("step - EndMeeting - 1.4 判断是主持人");
                    let channel_id = current_user_data_vec[1].clone();
                    if !channel_id.is_empty() {
                        println!("step - EndMeeting - 1.5 判断channel_id");
                        let users_to_notify = ONLINE_USERS
                            .iter()
                            .filter(|entry| entry.value().split(",").collect::<Vec<_>>()[1] == channel_id)
                            .map(|entry| entry.key().clone())
                            .collect::<Vec<_>>();
                        let clone_for_users_to_notify = users_to_notify.clone();
                        println!("step - EndMeeting - 1.6 判断channel_id 遍历循环");
                        for user_id in users_to_notify {
                            let user_data = ONLINE_USERS
                                .get(&user_id)
                                .map(|v| v.split(',').map(|s| s.to_string()).collect::<Vec<_>>());
                            if let Some(mut current_user_data_vec) = user_data {
                                current_user_data_vec[0] = "idle".to_string();
                                current_user_data_vec[1] = "".to_string();
                                current_user_data_vec[6] = "0".to_string();
                                let current_user_data_joined = current_user_data_vec.join(",");
                                println!("step - EndMeeting - 1.7 判断channel_id 遍历循环 1");
                                ONLINE_USERS.insert(user_id.clone(), current_user_data_joined.clone());
                                println!("step - EndMeeting - 1.8 判断channel_id 遍历循环 2");
                                tokio::spawn(async move {
                                    if let Err(e) = update_client_redis_data(&user_id, current_user_data_joined).await {
                                        println!("更新redis数据失败：{:?} 用户id {}", e, user_id);
                                    } else {
                                        println!("更新redis数据成功");
                                    }
                                });
                                println!("玄幻完毕");
                            }
                        }
                        println!("step - EndMeeting - 1.9 发送消息 遍历循环 3");
                        for user_id in clone_for_users_to_notify {
                            let target_sender = CLIENT_SENDERS
                                .iter()
                                .find(|entry| entry.key().0 == user_id)
                                .map(|entry| entry.key().clone());
                            if let Some(target_sender) = target_sender {
                                println!("step - EndMeeting 消息发送给 {:?}", &target_sender);
                                send_inside_message(
                                    &target_sender,
                                    event_sender,
                                    serde_json::json!({
                                        "msgType": "CmdEndMeeting",
                                        "fromID": "0",
                                        "fromName": "Server",
                                        "toID": user_id,
                                        "msgData": {
                                            "channelId": "",
                                            "rtcToken": ""
                                        }
                                    })
                                        .to_string(),
                                    &user_id,
                                )
                                    .await;
                            }
                        }
                        tokio::spawn(async move {
                            notify_all_clients_to_update_online_users().await;
                        });
                    }
                }
            }
        }
        "Hangup" => {
            println!("step - hangup -1 接收到hangup消息");
            let user_data = ONLINE_USERS
                .get(from_id)
                .map(|v| v.split(',').map(|s| s.to_string()).collect::<Vec<_>>());
            if let Some(mut current_user_data_vec) = user_data {
                let current_chatting_channel_id = current_user_data_vec[1].clone();
                println!("step - hangup -2 修改数据信息");
                current_user_data_vec[0] = "idle".to_string();
                current_user_data_vec[1] = "".to_string();
                current_user_data_vec[6] = "0".to_string();
                let current_user_data_joined = current_user_data_vec.join(",");
                println!("step - hangup -3 获取当前用户数据");
                ONLINE_USERS.insert(from_id.to_string(), current_user_data_joined.clone());
                println!("step - hangup -4 更新数据");
                let from_id_clone = from_id.clone();
                tokio::spawn(async move {
                    if let Err(e) =
                        update_client_redis_data(&from_id_clone, current_user_data_joined).await
                    {
                        println!("更新redis数据失败：{:?} 用户id {}", e, from_id_clone);
                    } else {
                        println!("更新redis数据成功");
                    }
                });
                send_inside_message(
                    &target_sender_which,
                    event_sender,
                    serde_json::json!({
                        "msgType": "CmdHangup",
                        "fromID": "0",
                        "fromName": "Server",
                        "toID": from_id,
                        "msgData": {
                            "channelId": "",
                            "rtcToken": ""
                        }
                    })
                        .to_string(),
                    from_id,
                )
                    .await;
                println!("step - hangup -5 发送消息");
                println!("step - hangup -6 发送消息 当前用户是主持人，需要更新数据");
                let remaining_users = ONLINE_USERS
                    .iter()
                    .filter(|entry| entry.value().split(',').collect::<Vec<_>>()[1] == current_chatting_channel_id)
                    .map(|entry| entry.key().clone())
                    .collect::<Vec<_>>();
                if remaining_users.is_empty() {
                    println!("当前频道没有人员，请重新发起通话");
                } else {
                    println!("准备发送CmdLeave消息");
                    for user_id in &remaining_users {
                        let sender_found = CLIENT_SENDERS
                            .iter()
                            .find(|entry| entry.key().0 == *user_id)
                            .map(|entry| entry.key().clone());
                        if let Some(target_sender) = sender_found {
                            let cmd_leave_message = serde_json::json!({
                                "msgType": "CmdLeave",
                                "fromID": from_id,
                                "fromName": "Unknown",
                                "toID": user_id,
                                "msgData": {}
                            })
                                .to_string();
                            send_inside_message(
                                &target_sender,
                                event_sender,
                                cmd_leave_message,
                                user_id,
                            )
                                .await;
                        }
                    }
                    println!("判断是否仅为1人，需要自动挂断");
                    if remaining_users.len() == 1 {
                        let channel_id_clone = current_chatting_channel_id.clone();
                        let ev_clone = event_sender.clone();
                        let remaining_user_id = remaining_users[0].clone();
                        let (cancel_tx, mut cancel_rx) = mpsc::unbounded_channel::<()>();
                        channel_hangup_procedure_map.insert(current_chatting_channel_id.clone(), cancel_tx);
                        tokio::spawn(async move {
                            tokio::select! {
                                _ = sleep(Duration::from_secs(15)) => {
                                    println!("20s内如果该频道与会人数仍为1，则对剩下来的用户发送CmdHangup指令 {:?}", remaining_user_id);
                                    let remain_user_data = ONLINE_USERS.get(&remaining_user_id).map(|v| v.split(',').map(|s| s.to_string()).collect::<Vec<_>>());
                                    if let Some(mut user_info_data_for_remained) = remain_user_data {
                                        let sender_d = CLIENT_SENDERS
                                            .iter()
                                            .find(|entry| entry.key().0 == remaining_user_id)
                                            .map(|entry| entry.key().clone());
                                        if let Some(target_sender_which) = sender_d {
                                            user_info_data_for_remained[0] = "idle".to_string();
                                            user_info_data_for_remained[1] = "".to_string();
                                            user_info_data_for_remained[6] = "0".to_string();
                                            let user_info_data_for_remained_joined = user_info_data_for_remained.join(",");
                                            println!("准备更新剩余一人数据！");
                                            ONLINE_USERS.insert(remaining_user_id.clone(), user_info_data_for_remained_joined.clone());
                                            println!("更新剩余一人数据成功！");
                                            let clone_for_redis = remaining_user_id.clone();
                                            tokio::spawn(async move {
                                                if let Err(e) = update_client_redis_data(&clone_for_redis, user_info_data_for_remained_joined).await {
                                                    println!("更新redis数据失败：{:?} 用户id {}", e, clone_for_redis);
                                                } else {
                                                    println!("更新redis数据成功");
                                                }
                                            });
                                            send_inside_message(
                                                &target_sender_which,
                                                &ev_clone,
                                                serde_json::json!({
                                                    "msgType": "CmdHangup",
                                                    "fromID": "0",
                                                    "fromName": "Server",
                                                    "toID": remaining_user_id,
                                                    "msgData": {
                                                        "channelId": "",
                                                        "rtcToken": ""
                                                    }
                                                }).to_string(),
                                                &remaining_user_id,
                                            )
                                            .await;
                                            tokio::spawn(async move {
                                                notify_all_clients_to_update_online_users().await;
                                            });
                                        }
                                    }
                                }
                                _ = cancel_rx.recv() => {
                                    println!("收到取消信号，有新的人员加入到会议，会议频道号 {} 结束线程", channel_id_clone);
                                }
                            }
                        });
                    } else {
                        let (allowed_users, disallowed_users): (Vec<String>, Vec<String>) =
                            remaining_users.into_iter().partition(|user_id| HOST_ENABLED_ID_SET.contains(user_id.as_str()));
                        let allowed_users_clone = allowed_users.clone();
                        if !allowed_users_clone.is_empty() {
                            if let Some(allowed_user_id) = allowed_users_clone.get(0) {
                                let allowed_user_data = ONLINE_USERS.get(allowed_user_id).map(|v| v.split(',').map(|s| s.to_string()).collect::<Vec<_>>());
                                if let Some(mut user_info_data_for_allowed) = allowed_user_data {
                                    user_info_data_for_allowed[6] = "1".to_string();
                                    let user_info_data_for_allowed_joined = user_info_data_for_allowed.join(",");
                                    ONLINE_USERS.insert(allowed_user_id.clone(), user_info_data_for_allowed_joined.clone());
                                    let allowed_user_id_clone = allowed_user_id.clone();
                                    tokio::spawn(async move {
                                        if let Err(e) = update_client_redis_data(&allowed_user_id_clone, user_info_data_for_allowed_joined).await {
                                            println!("更新redis数据失败：{:?} 用户id {}", e, allowed_user_id_clone);
                                        } else {
                                            println!("更新redis数据成功");
                                        }
                                    });
                                }
                            }
                        } else if !disallowed_users.is_empty() {
                            if let Some(disallowed_user_id) = disallowed_users.get(0) {
                                let disallowed_user_data = ONLINE_USERS.get(disallowed_user_id).map(|v| v.split(',').map(|s| s.to_string()).collect::<Vec<_>>());
                                if let Some(mut current_host_data) = disallowed_user_data {
                                    current_host_data[6] = "1".to_string();
                                    let current_host_data_joined = current_host_data.join(",");
                                    ONLINE_USERS.insert(disallowed_user_id.clone(), current_host_data_joined.clone());
                                    let disallowed_user_id_clone = disallowed_user_id.clone();
                                    tokio::spawn(async move {
                                        if let Err(e) = update_client_redis_data(&disallowed_user_id_clone, current_host_data_joined).await {
                                            println!("更新redis数据失败：{:?} 用户id {}", e, disallowed_user_id_clone);
                                        } else {
                                            println!("更新redis数据成功");
                                        }
                                    });
                                }
                            }
                        }
                    }
                }
                tokio::spawn(async move {
                    notify_all_clients_to_update_online_users().await;
                });
            }
        }
        "Connect" => {
            println!(
                "收到客户端Connect消息连接 {} 频道信息 {:?}",
                from_id, client_message_data
            );
            let to_id_user = &client_message_data.to_id;
            println!("Step Connect-1");
            let to_id_user_data = ONLINE_USERS
                .get(to_id_user)
                .map(|v| v.split(',').map(|s| s.to_string()).collect::<Vec<_>>());
            if let Some(data_split) = to_id_user_data {
                println!("Step Connect-1.1");
                let target_channel_id = data_split[1].clone();
                if !target_channel_id.is_empty() {
                    println!("Step Connect-1.2");
                    if let Some(hangup_personnel) = refuse_procedure_map.get(from_id) {
                        if let Err(e) = hangup_personnel.send(()) {
                            println!("挂断personnel的任务终结失败 {:?}", e);
                        } else {
                            println!("挂断personnel的任务已经发送成功");
                        }
                    }
                    println!("Step Connect-1.3");
                    if let Some(hangup_channel_task) = channel_hangup_procedure_map.get(&target_channel_id) {
                        if let Err(e) = hangup_channel_task.send(()) {
                            println!("挂断channel的任务终结失败 {:?}", e);
                        } else {
                            println!("挂断channel的任务已经发送成功");
                        }
                    }
                    println!("Step Connect-1.4");
                    let already_had_host = ONLINE_USERS
                        .iter()
                        .filter(|entry| entry.value().split(",").collect::<Vec<_>>()[1] == target_channel_id)
                        .any(|entry| entry.value().split(",").collect::<Vec<_>>()[6] == "1");
                    println!("Step Connect-1.5");
                    if already_had_host {
                        println!("Step Connect-1.6");
                        println!("当前channel已经有主持人，直接返回 {}", target_channel_id);
                    } else {
                        println!("当前channel没有主持人，开始处理 {}", target_channel_id);
                        println!("Step Connect-1.8");
                        let last_remaining_id_set = target_channel_id.split("_").map(|s| s.to_string()).collect::<Vec<_>>();
                        if let Some(channel_id_first_element) = last_remaining_id_set.get(0) {
                            println!("Step Connect-2");
                            if HOST_ENABLED_ID_SET.contains(channel_id_first_element.as_str()) {
                                let host_data = ONLINE_USERS.get(channel_id_first_element).map(|v| v.split(",").map(|s| s.to_string()).collect::<Vec<_>>());
                                if let Some(mut host_data_vec) = host_data {
                                    host_data_vec[6] = "1".to_string();
                                    let host_data_joined = host_data_vec.join(",");
                                    println!("Step Connect-3");
                                    println!("Step Connect-4");
                                    ONLINE_USERS.insert(channel_id_first_element.clone(), host_data_joined.clone());
                                    println!("Step Connect-5");
                                    let the_first_for = channel_id_first_element.clone();
                                    tokio::spawn(async move {
                                        if let Err(e) = update_client_redis_data(&the_first_for, host_data_joined).await {
                                            println!("更新redis数据失败：{:?} 用户id {} 成为主持人", e, the_first_for);
                                        } else {
                                            println!("更新redis数据成功");
                                        }
                                    });
                                    println!("Step Connect-6");
                                }
                            } else {
                                println!("Step Connect-7");
                                let all_in_group_chatters_id = ONLINE_USERS
                                    .iter()
                                    .filter(|entry| entry.value().split(",").collect::<Vec<_>>()[1] == target_channel_id)
                                    .map(|entry| entry.key().clone())
                                    .collect::<Vec<_>>();
                                let first_host_id = all_in_group_chatters_id
                                    .iter()
                                    .find(|id| HOST_ENABLED_ID_SET.contains(id.as_str()));
                                let mut id = String::new();
                                if let Some(target_id) = first_host_id {
                                    id = target_id.clone();
                                } else if let Some(first_element) = all_in_group_chatters_id.first() {
                                    id = first_element.clone();
                                }
                                println!("Step Connect-7.5 找到具备成为主持人的host-id");
                                let host_data = ONLINE_USERS.get(&id).map(|v| v.split(",").map(|s| s.to_string()).collect::<Vec<_>>());
                                if let Some(mut host_data_vec) = host_data {
                                    host_data_vec[6] = "1".to_string();
                                    println!("Step Connect-8");
                                    let host_data_joined = host_data_vec.join(",");
                                    println!("Step Connect-9");
                                    ONLINE_USERS.insert(id.clone(), host_data_joined.clone());
                                    println!("Step Connect-9.1");
                                    let host_id_clone = id.clone();
                                    tokio::spawn(async move {
                                        if let Err(e) = update_client_redis_data(&host_id_clone, host_data_joined).await {
                                            println!("更新redis数据失败：{:?} 用户id {} 成为主持人", e, host_id_clone);
                                        } else {
                                            println!("更新redis数据成功");
                                        }
                                    });
                                    println!("Step Connect-10");
                                }
                            }
                        }
                    }
                    println!("Step Connect-11");
                    let all_not_calling_users = ONLINE_USERS
                        .iter()
                        .filter(|entry| {
                            let entry_split = entry.value().split(",").collect::<Vec<_>>();
                            entry_split[1] == target_channel_id && entry_split[0] != "calling"
                        })
                        .map(|entry| entry.key().clone())
                        .collect::<Vec<_>>();
                    println!(
                        "Step Connect-11.5 当前所有channel用户 {:?}和channelID {}",
                        all_not_calling_users, target_channel_id
                    );
                    for user_id in all_not_calling_users {
                        let target_sender_which = CLIENT_SENDERS
                            .iter()
                            .find(|entry| entry.key().0 == user_id)
                            .map(|entry| entry.key().clone());
                        if let Some(target_sender_which) = target_sender_which {
                            let json_str = serde_json::json!({
                                "msgType": "CmdConnect",
                                "msgData": {
                                    "channelID": target_channel_id,
                                    "rtcToken": ""
                                },
                                "fromID": "0",
                                "fromName": "Server",
                                "toID": user_id
                            })
                                .to_string();
                            if let Err(e) = event_sender.send(Event::SendClientMessage(
                                target_sender_which.clone(),
                                json_str,
                                false,
                            )) {
                                println!("发送给用户id {} 独立消息失败：{:?}", user_id, e);
                            } else {
                                println!("发送给用户id {} 独立消息成功 CmdConnect", user_id);
                            }
                        }
                    }
                    tokio::spawn(async move {
                        notify_all_clients_to_update_online_users().await;
                    });
                    println!("Step Connect-11");
                } else {
                    if let Err(e) = event_sender.send(Event::SendClientMessage(
                        target_sender_which.clone(),
                        serde_json::json!({
                            "msgType": "Error",
                            "msgData": "对方数据出现异常，缺少channelID",
                            "fromID": "0",
                            "fromName": "Server",
                            "toID": from_id
                        })
                            .to_string(),
                        false,
                    )) {
                        println!("发送给用户id {} 独立消息失败：{:?}", from_id, e);
                    }
                }
            } else {
                if let Err(e) = event_sender.send(Event::SendClientMessage(
                    target_sender_which.clone(),
                    serde_json::json!({
                        "msgType": "Error",
                        "msgData": "必须传递to_id",
                        "fromID": "0",
                        "fromName": "Server",
                        "toID": from_id
                    })
                        .to_string(),
                    false,
                )) {
                    println!("发送给用户id {} 独立消息失败：{:?}", from_id, e);
                }
            }
        }
        "KickOut" => {
            println!("step - KickOut - 1 收到踢出通知");
            let user_found = ONLINE_USERS
                .get(from_id)
                .map(|v| v.split(",").map(|s| s.to_string()).collect::<Vec<_>>());
            if let Some(user_split) = user_found {
                if user_split[6] == "1" {
                    let current_cid = user_split[1].clone();
                    println!("step - KickOut - 2 释放资源");
                    let current_chatters = ONLINE_USERS
                        .iter()
                        .filter(|entry| entry.value().split(",").collect::<Vec<_>>()[1] == current_cid)
                        .map(|entry| entry.key().clone())
                        .collect::<Vec<_>>();
                    println!("step - KickOut - 3 获取同channelId的人");
                    for user_id in current_chatters {
                        let target_sender_which = CLIENT_SENDERS
                            .iter()
                            .find(|entry| entry.key().0 == user_id)
                            .map(|entry| entry.key().clone());
                        if let Some(target_sender_which) = target_sender_which {
                            if user_id == client_message_data.to_id {
                                println!("step - KickOut 4 --踢出对应消息的人");
                                let kicked_user_data = ONLINE_USERS.get(&user_id).map(|v| v.split(",").map(|s| s.to_string()).collect::<Vec<_>>());
                                if let Some(mut kicked_user_data) = kicked_user_data {
                                    kicked_user_data[0] = "idle".to_string();
                                    kicked_user_data[1] = "".to_string();
                                    kicked_user_data[6] = "0".to_string();
                                    let data_joined = kicked_user_data.join(",");
                                    println!("准备修改资源");
                                    ONLINE_USERS.insert(user_id.clone(), data_joined.clone());
                                    println!("修改资源成功！");
                                    let cloned_user_id = user_id.clone();
                                    tokio::spawn(async move {
                                        if let Err(e) = update_client_redis_data(&cloned_user_id, data_joined).await {
                                            println!("修改redis数据失败：{:?}", e);
                                        } else {
                                            println!("修改redis数据成功！");
                                        }
                                    });
                                    if let Err(e) = event_sender.send(Event::SendClientMessage(
                                        target_sender_which.clone(),
                                        serde_json::json!({
                                            "msgType": "CmdHangup",
                                            "msgData": {
                                                "channelID": current_cid,
                                                "rtcToken": ""
                                            },
                                            "fromID": "0",
                                            "fromName": "Server",
                                            "toID": user_id
                                        })
                                            .to_string(),
                                        false,
                                    )) {
                                        println!("发送给用户id {} 独立消息失败：{:?}", user_id, e);
                                    } else {
                                        println!("发送给用户id {} 独立消息成功 CmdHangup", user_id);
                                    }
                                }
                            } else {
                                if let Err(e) = event_sender.send(Event::SendClientMessage(
                                    target_sender_which.clone(),
                                    serde_json::json!({
                                        "msgType": "CmdKickOut",
                                        "msgData": {
                                            "channelID": current_cid,
                                            "rtcToken": ""
                                        },
                                        "toID": client_message_data.to_id,
                                        "fromName": "Server",
                                        "fromID": "0"
                                    })
                                        .to_string(),
                                    false,
                                )) {
                                    println!("发送给用户id {} 独立消息失败：{:?}", user_id, e);
                                } else {
                                    println!("发送给用户id {} 独立消息成功 CmdHangup", user_id);
                                }
                            }
                        }
                    }
                    tokio::spawn(async move {
                        notify_all_clients_to_update_online_users().await;
                    });
                } else {
                    println!("step - KickOut - 2 不是主持人");
                    if let Err(e) = event_sender.send(Event::SendClientMessage(
                        target_sender_which.clone(),
                        serde_json::json!({
                            "msgType": "Error",
                            "msgData": "只有主持人才能踢出用户",
                            "fromID": "0",
                            "fromName": "Server",
                            "toID": from_id
                        })
                            .to_string(),
                        false,
                    )) {
                        println!("发送给用户id {} 独立消息失败：{:?}", from_id, e);
                    } else {
                        println!("发送给用户id {} 独立消息成功 Error", from_id);
                    }
                }
            }
        }
        "Mute" => {
            println!("step - Mute - 1 收到静音通知");
            let to_id = client_message_data.to_id.clone();
            if to_id.is_empty() {
                if let Err(e) = event_sender.send(Event::SendClientMessage(
                    target_sender_which.clone(),
                    serde_json::json!({
                        "msgType": "Error",
                        "msgData": "必须传递to_id",
                        "fromID": "0",
                        "fromName": "Server",
                        "toID": from_id
                    })
                        .to_string(),
                    false,
                )) {
                    println!("发送给用户id {} 独立消息失败：{:?}", from_id, e);
                } else {
                    println!("发送给用户id {} 独立消息成功", from_id);
                }
            } else {
                let user_found = ONLINE_USERS
                    .get(from_id)
                    .map(|v| v.split(",").map(|s| s.to_string()).collect::<Vec<_>>());
                if let Some(user_data_now) = user_found {
                    if user_data_now[6] == "1" {
                        let channel_id = user_data_now[1].clone();
                        let all_chatters_id = ONLINE_USERS
                            .iter()
                            .filter(|entry| entry.value().split(",").collect::<Vec<_>>()[1] == channel_id)
                            .map(|entry| entry.key().clone())
                            .collect::<Vec<_>>();
                        let json_str = serde_json::json!({
                            "msgType": "CmdMute",
                            "msgData": {},
                            "fromID": to_id,
                            "fromName": "Server",
                            "toID": to_id
                        })
                            .to_string();
                        for user_id in all_chatters_id {
                            let target_sender_which = CLIENT_SENDERS
                                .iter()
                                .find(|entry| entry.key().0 == user_id)
                                .map(|entry| entry.key().clone());
                            if let Some(target_sender_which) = target_sender_which {
                                if let Err(e) = event_sender.send(Event::SendClientMessage(
                                    target_sender_which.clone(),
                                    json_str.clone(),
                                    false,
                                )) {
                                    println!("发送给用户id {} 独立消息失败：{:?}", user_id, e);
                                } else {
                                    println!("发送给用户id {} 独立消息成功", user_id);
                                }
                            }
                        }
                    } else {
                        if let Err(e) = event_sender.send(Event::SendClientMessage(
                            target_sender_which.clone(),
                            serde_json::json!({
                                "msgType": "Error",
                                "msgData": "您不是主持人，无法静音他人",
                                "fromID": "0",
                                "fromName": "Server",
                                "toID": from_id
                            })
                                .to_string(),
                            false,
                        )) {
                            println!("发送给用户id {} 独立消息失败：{:?}", from_id, e);
                        } else {
                            println!("发送给用户id {} 独立消息成功", from_id);
                        }
                    }
                }
            }
        }
        "MuteAll" => {
            println!("step - MuteAll - 1 收到静音全部通知");
            let user_found = ONLINE_USERS
                .get(from_id)
                .map(|v| v.split(",").map(|s| s.to_string()).collect::<Vec<_>>());
            if let Some(current_user_data) = user_found {
                println!("step - MuteAll -2 拆解数据");
                if current_user_data[6] == "1" {
                    println!("step - MuteAll -3 静音全部可以，具备权限");
                    let current_channel_id = current_user_data[1].clone();
                    let all_users_in_channel = ONLINE_USERS
                        .iter()
                        .filter(|entry| entry.value().split(",").collect::<Vec<_>>()[1] == current_channel_id)
                        .map(|entry| entry.key().clone())
                        .collect::<Vec<_>>();
                    println!("step - MuteAll -4 筛选出所有channelId下的所有用户");
                    for user_id in all_users_in_channel {
                        let target_sender_which = CLIENT_SENDERS
                            .iter()
                            .find(|entry| entry.key().0 == user_id)
                            .map(|entry| entry.key().clone());
                        if let Some(target_sender_which) = target_sender_which {
                            let json_str = serde_json::json!({
                                "msgType": "CmdMuteAll",
                                "msgData": {
                                    "channelID": current_channel_id,
                                    "rtcToken": ""
                                },
                                "fromID": "0",
                                "fromName": "Server",
                                "toID": user_id
                            })
                                .to_string();
                            if let Err(e) = event_sender.send(Event::SendClientMessage(
                                target_sender_which.clone(),
                                json_str,
                                false,
                            )) {
                                println!("发送给用户id {} 独立消息失败：{:?}", user_id, e);
                            } else {
                                println!("发送给用户id {} 独立消息成功 CmdMuteAll", user_id);
                            }
                        }
                    }
                } else {
                    if let Err(e) = event_sender.send(Event::SendClientMessage(
                        target_sender_which.clone(),
                        serde_json::json!({
                            "msgType": "Error",
                            "msgData": "只有主持人才有操作权限",
                            "fromID": "0",
                            "fromName": "Server",
                            "toID": from_id
                        })
                            .to_string(),
                        false,
                    )) {
                        println!("发送给用户id {} 独立消息失败：{:?}", from_id, e);
                    } else {
                        println!("发送给用户id {} 独立消息成功", from_id);
                    }
                }
            }
        }
        "MuteSelf" | "UnMuteSelf" => {
            println!("step - MuteSelf-UN - 1 收到开关静音自己通知");
            let user_found = ONLINE_USERS
                .get(from_id)
                .map(|v| v.split(",").map(|s| s.to_string()).collect::<Vec<_>>());
            if let Some(data_split) = user_found {
                let channel_id = data_split[1].clone();
                let chatting_id_set = ONLINE_USERS
                    .iter()
                    .filter(|entry| entry.value().split(",").collect::<Vec<_>>()[1] == channel_id)
                    .map(|entry| entry.key().clone())
                    .collect::<Vec<_>>();
                println!("step -2 -Mute-orNot 准备广播消息");
                for chatting_id in chatting_id_set {
                    let target_sender_which = CLIENT_SENDERS
                        .iter()
                        .find(|entry| entry.key().0 == chatting_id)
                        .map(|entry| entry.key().clone());
                    if let Some(target_sender_which) = target_sender_which {
                        if let Err(e) = event_sender.send(Event::SendClientMessage(
                            target_sender_which.clone(),
                            serde_json::json!({
                                "msgType": client_message_data.msg_type.as_str(),
                                "msgData": {},
                                "fromID": from_id,
                                "fromName": "Unknown",
                                "toID": chatting_id
                            })
                                .to_string(),
                            false,
                        )) {
                            println!("发送给用户id {} 独立消息失败：{:?}", chatting_id, e);
                        }
                    }
                }
            }
        }
        _ => {}
    }
}